﻿local addonName, addonTable = ...
local L = addonTable.L
local addon = PriorityList
local AceGUI = LibStub("AceGUI-3.0")

-- string colors
local LIGHT_BLUE  = "|cff00ccff"
local LIGHT_RED   = "|cffff6060"
local GREEN       = "|cff71C671"
local LIGHT_GREEN = "|cff71FF71"
local YELLOW      = "|cffffff00"
local GREY        = "|cff888888"
local GOLD        = "|cffffcc00"
local RED         = "|cffff0000"
local WHITE       = "|cffffffff"
local CHOCOLATE   = "|cffCD661D"
local function AddColor(str,color)
 return color..str.."|r"
end

addon.opt = {}

function addon:IsLooted(itemID)
 if itemID < 0 then 
  return true
 else
  return false
 end
end

function addon:SetLooted(char,itemID,flag,refresh)
local pos = addon:GetItemPos(char,itemID)
 if PLWish_List[char] then
  if flag == false then
   PLWish_List[char][pos] = abs(PLWish_List[char][pos])
  else
   PLWish_List[char][pos] = -abs(PLWish_List[char][pos])
  end
  if refresh then
   addon:GenerateItemsList("main")
   addon:PopulateItemsList_new()
  end
 end
end

function addon:GetItemPos(name,itemID)
local result = table.getn(PLWish_List[name])
 for index,ID in pairs(PLWish_List[name]) do
  if ID == itemID or ID == -itemID then result = index end
 end
 return result
end

function addon:AddItem(name,itemID,refresh,position)
if table.getn(addon.import_var.Wish_List) == 0 then
 if not PLWish_List[name] then
  PLWish_List[name] = {}
 end
 if PLWish_List[name] then
  if not addon:TableMatch(PLWish_List[name],itemID,true) then
   if position and position <= table.getn(PLWish_List[name]) then
    table.insert(PLWish_List[name],position,itemID)
   else
    table.insert(PLWish_List[name],itemID)
   end
   addon:SetStatus(string.format(L["st_additem"],itemID,name)) --sysmess
  end
  if refresh then
   addon:GenerateItemsList("main")
   addon:PopulateItemsList_new()
  end
 end
else
 addon:SetStatus(L["listdisabled"],smc_r,smc_g,smc_b)
end
end

function addon:DeleteItem(itemID, refresh, charonly, name)
 local tmp_ID = nil
 local delitem = function()
  if charonly then
   if PLWish_List[name] then
    for i,ID in pairs(PLWish_List[name]) do
     if ID == itemID then
      tmp_ID = i
      break
     end
    end
   if tmp_ID ~= nil then
    table.remove(PLWish_List[name],tmp_ID)
   end
  end
 else
  table.foreach(PLWish_List,
   function(k,v) --foreach
    for i,ID in pairs(v) do
     if ID == itemID or ID == -itemID then
      tmp_ID = i 
      break
     end
    end
    if tmp_ID ~= nil then
     table.remove(v,tmp_ID) 
    end
   end)
 end
 if refresh then
  addon:GenerateItemsList("main")
  addon:PopulateItemsList_new()
 end
 end
 -- if it doesn't refresh the list, it's a way to change an itemid and it doesn't require confirms
 if refresh then addon:ConfirmCheck(delitem) else delitem() end
end

function addon:PopulateItemsList_new()
local LISTEMPTY_ITEM = {itemID = 999997,
                        name_col = L["listempty"],
                        link = "nolink",
                        flag = "msg",
                        spacing = 0}

 local list = ""
 if addon.opt.tab_selected == L["itemlist"] then -- items list tab
  list = "items"
 elseif addon.opt.tab_selected == L["opt_bosslist"] then -- items list tab
  list = "bosslist"
 elseif addon.opt.tab_selected == L["opt_Import/Export"] then -- import list tab
  list = "import"
 elseif addon.opt.tab_selected == L["opt_wish"] then -- wish list tab
  list = "wishlist"
 end
 if list ~= "" then
  if addon.opt.frm_items then
   addon.opt.frm_items:ReleaseChildren()
  end
  local found = false
  if addon:SortItemsList(addon.opt.edt_search:GetText(),list) then
   found = true
  end
  if not found then --list empty
   addon:NewButton(LISTEMPTY_ITEM,list)
  end
 end
end

-- items list tab creation
function addon:TabInit_ItemsList(container)
if container then

 addon.opt.edt_search:Show()
 addon.opt.edt_search_clear:Show()
 addon.opt.frm_items = AceGUI:Create("ScrollFrame")
 addon.opt.frm_items:SetFullWidth(true)
 addon.opt.frm_items:SetFullHeight(true)
 addon.opt.frm_items:SetLayout("List")
 container:AddChild(addon.opt.frm_items)

 addon:PopulateItemsList_new()
end
end
-- items list tab creation end

function addon:SortItemsList(filter,list)
local CHAR_ID   = 999999
local IMPORT_ID = 999998
local BOSS_ID   = 999995
local BOSS_SPACER_ID = 999996
local LISTEMPTY_ID = 999997
local result = false
local ITEMS_NOT_IN_SM = {itemID = LISTEMPTY_ID,
                        name_col = L["nolistinsimplemode"],
                        link = "nolink",
                        flag = "msg",
                        spacing = 0}

 --indexed foreach
 local function foreachinorder(t,f)
  local keys = {}
  for k,_ in pairs(t) do
    keys[#keys+1] = k
  end
  table.sort(keys,function (a,b)
   return string.lower(a) < string.lower(b)
  end)
  for _,k in ipairs(keys) do
   f(k,t[k])
  end
 end
--

 local function InsertItem(k,v,ischild,spacing,flag,sender)
  if list == "import" and k == 0 then --it's a "new item"
   -- do nothing
  else
   local found_name = false
   local found_need = false
   local needers,col_needers = addon:MergeNames(v)
   local itemName,itemLink,itemTexture = addon:Extr_ItemProp(k)
   local col_name = itemName
   local ID = k
   local looted = false
   if ID < 0 then
    ID = abs(ID)
    looted = true
   end

   if col_name:lower():find(filter) then found_name = true end
   if needers:lower():find(filter) then found_need = true end
   if found_name or found_need or list ~= "items" then
    if found_name and filter ~= "" and filter ~= " " then
     col_name = string.gsub(col_name:lower(),filter,AddColor(filter,LIGHT_RED)..GOLD)
    end
    col_name = AddColor(col_name,GOLD)
    if not addon:IsSimpleMode() then
     if found_need and filter ~= "" and filter ~= " " then
      col_needers = string.gsub(col_needers:lower(),filter,AddColor(filter,LIGHT_RED)..GREY)
     end
     col_needers = AddColor(col_needers,GREY) --needers color also in MergeNames()
    else
     col_needers = ""
    end

    if flag == "add" then
     col_name = AddColor(L["New:"].." ", LIGHT_GREEN)..col_name
    elseif flag == "del" then
     col_name = AddColor(L["Will be deleted:"].." ", LIGHT_RED)..col_name
    elseif flag and flag:find("mod") then --mod flags in the same string
     col_name = AddColor(L["Changed:"].." ", LIGHT_BLUE)..col_name
    end
    addon:NewButton({itemID = ID,
                                    looted = looted,
                                    name = itemName,
                                    name_col = col_name,
                                    link = itemLink,
                                    texture = itemTexture,
                                    needs = needers,
                                    needs_col = col_needers,
                                    flag = flag,
                                    child = ischild,
                                    spacing = spacing,
                                    sender = sender},list)
   else
  --
   end
  end
 end

 local function InsertChar(name,items,sender)
  local child   = ""
  local spacing = 0

   if name:lower():find(filter) then
    --insert char
    if list == "import" then
     child = sender
     spacing = 20
    end

    addon:NewButton({itemID = CHAR_ID,
                                    name = name,
                                    name_col = AddColor(name,GREEN),
                                    link = "nolink",
                                    needs_col = AddColor(tostring(table.getn(items)).." "..L["itemsinlist"],LIGHT_GREEN),
                                    flag = "char",
                                    child = child,
                                    spacing = spacing},list)
    --insert char's items
    if list == "import" then
     local exists = false
     table.foreach(items,
      function(k,v)
       if PLWish_List[name] ~= nil then
        exists = false
        table.foreach(PLWish_List[name], function(index,id)
         local changed = false
         local cng = "mod"
         if id == v or id == -v then --already exists
          if (addon:IsLooted(v) and not addon:IsLooted(id)) then
           changed = true
           cng = cng..AddColor("("..L["Lootedflag"].." "..AddColor(L["added"],LIGHT_GREEN),WHITE)..") "
          elseif (addon:IsLooted(id) and not addon:IsLooted(v)) then
           changed = true
           cng = cng..AddColor("("..L["Lootedflag"].." "..AddColor(L["removed"],LIGHT_RED),WHITE)..") "
          end
          if (k > index) then
           changed = true
           cng = cng..AddColor(" -"..k-index.." ",LIGHT_RED)
          elseif (k < index) then
           changed = true
           cng = cng..AddColor(" +"..index-k.." ",LIGHT_GREEN)
          end
          if changed then
           InsertItem(v,addon.import_var.Items_List[v][sender],name,40,cng,sender)
          else
           InsertItem(v,addon.import_var.Items_List[v][sender],name,40,"",sender)
          end
          exists = true
         end
        end)
        else -- char doesn't exist
         InsertItem(v,addon.import_var.Items_List[v][sender],name,40,"add",sender)
         exists = true
        end
        if not exists then
         InsertItem(v,addon.import_var.Items_List[v][sender],name,40,"add",sender)
        end
      end)
    else -- list == wishlist or bosslist
     table.foreach(items,
     function(k,v)
      InsertItem(v,Items_List[v],name,20,"")
     end)
    end
   end
  end

 local function InsertPlayer(name,items)
                                  addon:NewButton({itemID = IMPORT_ID,
                                  name = name,
                                  name_col = AddColor(name,LIGHT_BLUE),
                                  link = "nolink",
                                  flag = "char",
                                  spacing = 0},list)
 end

 
 local function InsertBoss(name,items)
  local needers = ""
  local col_needers = ""
  local tmp_icon = "Interface\\Common\\Indicator-Gray.blp"
  if addon:bl_bossopened(name) then tmp_icon = "Interface\\Common\\Indicator-Green.blp" end
  addon:NewButton({itemID = BOSS_ID,
                   name = name,
                   name_col = AddColor(name,CHOCOLATE),
                   link = "nolink",
                   flag = "msg",
                   icon = tmp_icon,
                   spacing = 0},list)
  foreachinorder(items,function(bossname,j)
   tmp_icon = "Interface\\Common\\Indicator-Gray.blp"
   if addon:bl_bossopened(name) then
   if addon:bl_bossopened(name,bossname) then tmp_icon = "Interface\\Common\\Indicator-Green.blp" end
   local b_icon = ""
   if j["icon"] then b_icon = j["icon"] end
    addon:NewButton({itemID = BOSS_ID,
                   name = bossname,
                   name_col = AddColor(bossname,CHOCOLATE),
                   bossicon = b_icon,
                   icon = tmp_icon,
                   link = "nolink",
                   raid = name,
                   flag = "msg",
                   spacing = 10},list)
  for diff,tab in pairs(j) do
   if diff ~= "icon" then
    tmp_icon = "Interface\\Common\\Indicator-Gray.blp"
   if addon:bl_bossopened(name,bossname) then
    if addon:bl_bossopened(name,bossname,diff) then tmp_icon = "Interface\\Common\\Indicator-Green.blp" end
       addon:NewButton({itemID = BOSS_ID,
                    name = diff,
                    name_col = AddColor(diff,LIGHT_RED),
                    icon = tmp_icon,
                    link = "nolink",
                    raid = name,
                    boss = bossname,
                    flag = "msg",
                    spacing = 20},list)
   if addon:bl_bossopened(name,bossname,diff) then
   table.foreach(tab,function(k,v)
    local itemName,itemLink,itemTexture = addon:Extr_ItemProp(v)
    local needers = ""
    local needers_col = ""
    if not addon:IsSimpleMode() then
     if Items_List[v] then needers,needers_col = addon:MergeNames(Items_List[v]) end
    else
     local p = UnitName("player")
     if addon:TableMatch(PLWish_List[p],v,true) then needers_col = AddColor(p,GREY) end
    end
    if v == -1 then
     addon:NewButton({itemID = BOSS_SPACER_ID,
                      looted = false,
                      name = "",
                      name_col = L["boss_spacer"],
                      link = "nolink",
                      texture = "",
                      spacing = 30,
                      flag   = "msg",
                      sender = ""},list)
    else
     addon:NewButton({itemID = v,
                      looted = false,
                      name = itemName,
                      name_col = AddColor(itemName,GOLD),
                      link = itemLink,
                      texture = itemTexture,
                      spacing = 30,
                      sender = AddColor(needers_col,GREY)},list)
    end
   end)
 end end end --bossopened
  end 
  end -- for do
  end)-- foreachinorder
 end

 filter = filter:lower()
 local mainlist = {}
 if    list == "items" then mainlist = Items_List
elseif list == "bosslist" then mainlist = PriorityList.boss_drops_table
elseif list == "import" then mainlist = addon.import_var.Wish_List
elseif list == "wishlist" then mainlist = PLWish_List
 end
 if list == "wishlist" then
  if addon:IsSimpleMode() then
   local p = UnitName("player")
   if mainlist[p] then 
    if not result then result = true end
    InsertChar(p,mainlist[p]) 
   end
  else--officer mode
  foreachinorder(mainlist,
   function(k,v)
    if not result then result = true end
    if type(k) == "string" then
      InsertChar(k,v)
    elseif type(k) == "number" then
     InsertItem(k,v)
    end
   end)
  end --simple mode check
 elseif list == "bosslist" then
  foreachinorder(mainlist,
   function(k,v)
    if not result then result = true end
    InsertBoss(k,v)
   end)
 elseif list == "items" then
  if addon:IsSimpleMode() then
   if not result then result = true end
   addon:NewButton(ITEMS_NOT_IN_SM,list)
  else--officer mode
  table.foreach(mainlist,
   function(k,v)
    if not result then result = true end
    if type(k) == "string" then
      InsertChar(k,v)
    elseif type(k) == "number" then
     InsertItem(k,v)
    end
   end)
  end
 else -- import list
  table.foreach(mainlist,function(k,v)
   if not result then result = true end
   InsertPlayer(k,v)
   foreachinorder(v,function(i,j)
    InsertChar(i,j,k)
    -- pending deletion items
    if PLWish_List[i] then
    table.foreach(PLWish_List[i],
     function(index,itemID)
      if addon.import_var.Deleting_List[k] and addon.import_var.Deleting_List[k][i] then
       if addon:TableMatch(addon.import_var.Deleting_List[k][i],itemID,true) then
        InsertItem(itemID,Items_List[itemID],name,40,"del",k)
       end
      end
     end)
     end
    end)
  end)
 end--lists

 return result
end